# -*- coding:UTF-8 -*-
import os
import tensorflow as tf
from tensorflow.keras.applications.mobilenet import MobileNet
from tensorflow.keras.layers import Dense
from tensorflow.keras import Model, optimizers
from tensorflow.keras import backend as K

species_num=5   #植物种类
learning_rate = 0.0001  #学习率
batch_size = 32 #每个batch大小
nb_epoch = 5 #迭代次数
data_path = './PlantsData'  #数据集路径
def load_data(path, set):
    def _parse_function(filename, label):
        image_string = tf.read_file(filename)
        image_decoded = tf.cast(tf.image.decode_image(image_string), dtype=tf.float32) / tf.constant(255, dtype=tf.float32)
        label = tf.one_hot(label, species_num)
        return image_decoded, label

    file_names = []
    labels = []
    for inds, species in enumerate(sorted(os.listdir(os.path.join(path, set)))):
        for img in os.listdir(os.path.join(path, set, species)):
            file_names.append(os.path.join(path, set, species, img))
            labels.append(inds)
    dataset = tf.data.Dataset.from_tensor_slices((tf.constant(file_names), tf.constant(labels)))
    dataset = dataset.map(_parse_function)
    return dataset, len(file_names)


def main():
    #加载先前训练好的模型
    init_model = tf.keras.applications.mobilenet.MobileNet(input_shape=(224, 224, 3), include_top=False,
                                                           weights='imagenet',
                                                           pooling='avg')
    x = tf.keras.layers.Dense(512, activation='relu')(init_model.output)
    output = tf.keras.layers.Dense(species_num, activation='softmax')(x)
    model = tf.keras.Model(inputs=init_model.input, outputs=output)

    optimizer = tf.keras.optimizers.SGD(learning_rate, momentum=0.9, nesterov=True, decay=0.0003)
    model.compile(optimizer=optimizer, loss='categorical_crossentropy', metrics=['accuracy'])
    model.load_weights('./models/checkpoints/model')

    test_dataset, test_num = load_data(data_path, 'test')
    test_dataset = test_dataset.batch(1)
    print ('data prepared! test: %d' % (test_num))
    #测试模型
    result = model.evaluate(test_dataset, steps=test_num, verbose=0)
    print ('test loss: %0.4f, test acc: %0.4f' % (result[0], result[1]))


if __name__ == '__main__':
    main()